/**
 * PETALS: PETALS Services Platform Copyright (C) 2005 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 * 
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.petals.ant.task;

import java.io.FileInputStream;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.management.Attribute;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.IntrospectionException;
import javax.management.InvalidAttributeValueException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.ow2.petals.ant.AbstractInstallerAntTask;

/**
 * Ant task : jbi-install-component. This task install a JBI component (service
 * engine of binding component) into the JBI environment.
 * 
 * @author ddesjardins chamerling - eBMWebsourcing
 */
public class InstallComponentTask extends AbstractInstallerAntTask {

    private final static Hashtable<String, Class<?>> primitiveClasses = new Hashtable<String, Class<?>>(
            12);

    static {
        primitiveClasses.put(Boolean.class.getName(), Boolean.class);
        primitiveClasses.put(Boolean.TYPE.getName(), Boolean.class);
        primitiveClasses.put(Short.class.getName(), Short.class);
        primitiveClasses.put(Boolean.TYPE.getName(), Short.class);
        primitiveClasses.put(Integer.class.getName(), Integer.class);
        primitiveClasses.put(Integer.TYPE.getName(), Integer.class);
        primitiveClasses.put(Long.class.getName(), Long.class);
        primitiveClasses.put(Long.TYPE.getName(), Long.class);
        primitiveClasses.put(Float.class.getName(), Float.class);
        primitiveClasses.put(Float.TYPE.getName(), Float.class);
        primitiveClasses.put(Double.class.getName(), Double.class);
        primitiveClasses.put(Double.TYPE.getName(), Double.class);
    }

    /**
     * The nested elements <param>
     */
    private List<Param> nestedParams = new ArrayList<Param>();

    /**
     * Location of the nestedParams file that contains MBean properties
     */
    private String params;

    // The list of attributes exposed by the Installer Configuration MBean
    private MBeanAttributeInfo[] attributeInfos;

    /**
     * Creates a new instance of {@link InstallComponentTask}
     * 
     */
    public InstallComponentTask() {
        super();
    }

    /**
     * Execute the ant task
     * 
     * @throws IOException
     */
    public void doTask() throws Exception {
        MBeanServerConnection connection = connector.getMBeanServerConnection();

        Object result = performAction(connection, AbstractInstallerAntTask.LOAD_NEW_INSTALLER);
        if (result instanceof ObjectName) {
            ObjectName installerName = (ObjectName) result;

            // Get the installer configuration
            Object result1 = connection.getAttribute(installerName, "InstallerConfigurationMBean");
            System.err.println("Getting the InstallerConfigurationMBean:" + result1);
            System.err.println(String.valueOf(nestedParams.size()) + " parameters to be set");
            if (result1 instanceof ObjectName) {
                // set the properties and parameters
                if (nestedParams.size() > 0 || params != null) {
                    getConfigurationAttributes(connection, (ObjectName) result1);
                    setConnectionAttributeFromPropertiesFile(connection, (ObjectName) result1);
                    setConnectionAttributeFromNestedParam(connection, (ObjectName) result1);
                }
            }

            // Install the component
            Object[] objects = new Object[0];
            String[] strings = new String[0];
            connection.invoke(installerName, "install", objects, strings);
            try {
                log("The component '" + installerName.getKeyProperty("name")
                        + "' has been installed");
            } catch (NullPointerException e) {
            }

        }
    }

    /**
     * Get the properties from the params file.
     * 
     * @return
     * @throws IOException
     */
    private Properties loadProperties() throws IOException {
        Properties props = new Properties();
        if (params != null)
            props.load(new FileInputStream(params));

        return props;
    }

    /**
     * Set the connection attributes from the properties file
     * 
     * @nestedParams connection
     * @nestedParams configurationMBean
     * @throws Exception
     */
    private void setConnectionAttributeFromPropertiesFile(MBeanServerConnection connection,
            ObjectName configurationMBean) throws Exception {
        Properties props = loadProperties();
        for (Map.Entry<Object, Object> entry : props.entrySet()) {
            System.err.println("Setting attribute from properties " + (String) entry.getKey()
                    + " = " + (String) entry.getValue());
            setAttribute(connection, configurationMBean, (String) entry.getKey(), (String) entry
                    .getValue());
        }
    }

    /**
     * Set the connection attribute from the nested params
     * 
     * @nestedParams connection
     * @nestedParams configurationMBean
     * @throws Exception
     */
    private void setConnectionAttributeFromNestedParam(MBeanServerConnection connection,
            ObjectName configurationMBean) throws Exception {
        // Set the properties of the <Param> element
        for (Param p : nestedParams) {
            System.err.println("Setting attribute " + p.getName() + " = " + p.getValue());
            setAttribute(connection, configurationMBean, p.getName(), p.getValue());
        }
    }

    private void setAttribute(MBeanServerConnection connection, ObjectName configurationMBean,
            String key, String value) throws InstanceNotFoundException, AttributeNotFoundException,
            InvalidAttributeValueException, MBeanException, ReflectionException, IOException {
        Object castedValue = null;
        for (MBeanAttributeInfo attributeInfo : this.attributeInfos) {
            if (attributeInfo.getName().equals(key)) {
                if (primitiveClasses.containsKey(attributeInfo.getType())) {
                    try {
                        Class<?> primitiveClass = primitiveClasses.get(attributeInfo.getType());
                        Method valueOfMethod = primitiveClass.getMethod("valueOf",
                                new Class[] { String.class });
                        castedValue = valueOfMethod.invoke(null, value);
                    } catch (SecurityException e) {

                    } catch (NoSuchMethodException e) {
                        throw new RuntimeException(e);
                    } catch (IllegalArgumentException e) {
                        throw new RuntimeException(e);
                    } catch (IllegalAccessException e) {
                        throw new RuntimeException(e);
                    } catch (InvocationTargetException e) {
                        throw new RuntimeException(e);
                    }
                } else if (attributeInfo.getType().equals(String.class)) {
                    castedValue = value;
                }
            }
        }
        if (castedValue == null) {
            throw new RuntimeException("Failed to identify primitive type of parameter '" + key
                    + "'");
        }
        Attribute attribute = new Attribute(key, castedValue);
        connection.setAttribute(configurationMBean, attribute);
    }

    private void getConfigurationAttributes(MBeanServerConnection connection,
            ObjectName configurationMBean) throws InstanceNotFoundException,
            IntrospectionException, ReflectionException, IOException {
        MBeanInfo mbeanInfo = connection.getMBeanInfo(configurationMBean);
        this.attributeInfos = mbeanInfo.getAttributes();
    }

    /**
     * @return the params
     */
    public String getParams() {
        return params;
    }

    /**
     * @nestedParams params the params to set
     */
    public void setParams(String params) {
        this.params = params;
    }

    /**
     * 
     * @return
     */
    public Param createParam() {
        Param param = new Param();
        nestedParams.add(param);
        return param;
    }

    /**
     * Parameter class for nested <param> elements
     * 
     * @author chamerling
     * 
     */
    public static class Param {
        /**
         * Name attribute
         */
        private String name;

        /**
         * Value attribute
         */
        private String value;

        /**
         * Creates a new instance of {@link Param}
         * 
         */
        public Param() {
            this("", "");
        }

        /**
         * Creates a new instance of {@link Param}
         * 
         * @param name
         * @param value
         */
        public Param(String name, String value) {
            this.name = name;
            this.value = value;
        }

        /**
         * @return the name
         */
        public String getName() {
            return name;
        }

        /**
         * @param name
         *      the name to set
         */
        public void setName(String name) {
            this.name = name;
        }

        /**
         * @return the value
         */
        public String getValue() {
            return value;
        }

        /**
         * @param value
         *      the value to set
         */
        public void setValue(String value) {
            this.value = value;
        }
    }
}
